import org.COPASI.CDataModel;

import java.util.Vector;

import org.COPASI.*;

//
public class ScanSnippet {
	public static void main(String[] args) {

		CDataModel dataModel = CRootContainer.addDatamodel();
		dataModel.loadFromFile("Alcohol_ss_model.xml");

		String delim = "\t";

		CModel model = dataModel.getModel();

		CScanTask scanTask = (CScanTask) dataModel.getTask("Scan");

		CScanProblem scanProblem = (CScanProblem) scanTask.getProblem();

		scanProblem.setSubtask(CTaskEnum.Task_steadyState);
		scanProblem.setOutputInSubtask(false);

		String item = "[sorafenib]_0";

		String cn = addScanItem(scanProblem, item, dataModel);

		CDataHandler dh = new CDataHandler();

		// i'll manually create a header, so it becomes easier
		// to interpet the output
		Vector<String> header = new Vector<String>();

		// add scan item to output and header
		dh.addDuringName(new CRegisteredCommonName(cn));
		header.add(item);

		// go through reactions to add fluxes
		for (int i = 0; i < model.getNumReactions(); i++) {
			CDataObject obj = model.getReaction(i);

			if (obj instanceof CMetab) {
				obj = ((CMetab) obj).getConcentrationReference();
			} else if (obj instanceof CReaction) {
				obj = ((CReaction) obj).getFluxReference();

			}

			// add as during handler to collect during the execution of the task
			dh.addDuringName(new CRegisteredCommonName(obj.getCN().getString()));
			// add to header
			header.add(obj.getObjectDisplayName());
		}

		// initialize passing along the output handler
		if (!scanTask.initializeRawWithOutputHandler((int) CCopasiTask.OUTPUT_UI, dh)) {
			System.err.println("Couldn't initialize the steady state task");
			System.err.println(CCopasiMessage.getAllMessageText());
		}
		// run
		if (!scanTask.processRaw(true)) {
			System.err.println("Couldn't run the steady state task");
			System.err.println(CCopasiMessage.getAllMessageText());
		}
		scanTask.restore();

		// at this point we have the output we collected, lets just print it

		// first the header
		for (int j = 0; j < header.size(); j++) {
			System.out.print(header.get(j));
			if (j + 1 < header.size())
				System.out.print(delim);
		}
		System.out.println();

		// get number of rows recorded
		int numRows = dh.getNumRowsDuring();

		// print each row collected during the run
		for (int i = 0; i < numRows; i++) {
			FloatStdVector data = dh.getNthRow(i);
			for (int j = 0; j < data.size(); j++) {
				System.out.print(data.get(j));
				if (j + 1 < data.size())
					System.out.print(delim);
			}
			System.out.println();
		}
		System.out.println();
	}

	/**
	 * add scan item, returning the cn for later
	 * 
	 * @param scanProblem problem
	 * @param item        display name of item
	 * @param dataModel   data model to find item in
	 * @return cn of reference found, otherwise null
	 */
	public static String addScanItem(CScanProblem scanProblem, String item, CDataModel dataModel) {

		int type = CScanProblem.SCAN_LINEAR;

		int numSteps = 10;

		CCopasiParameterGroup cItem = scanProblem.addScanItem(type, numSteps);

		double finalMin = 0.0;

		double finalMax = 5.0;

		CDataObject scanObj = dataModel.findObjectByDisplayName(item);

		if (scanObj == null) {
			System.err.println("couldn't resolve displayName: " + item);
			return null;
		}

		if (scanObj instanceof CModelEntity) {
			// resolve model elements to their initial value reference
			scanObj = ((CModelEntity) scanObj).getInitialValueReference();
		} else if (scanObj instanceof CCopasiParameter) {
			// resolve local parameters to its value reference
			scanObj = ((CCopasiParameter) scanObj).getValueReference();
		}

		String cn = scanObj.getCN().getString();

		cItem.getParameter("Maximum").setDblValue(finalMax);
		cItem.getParameter("Minimum").setDblValue(finalMin);
		cItem.getParameter("Object").setCNValue(cn);

		return cn;
	}

}
